import java.util.*;


/**
 * The Communication Supervision Layer class.
 * @author Davide Basile (ISTI CNR Italy)
 * @version 1.0
 * @created 23-Nov-2020 11:13:45 AM
 */
public class CSL implements StateMachineContext {

	/**
	 * Accumulating time since last SAI_CONNECT_REQUEST. 
	 */
	private int connect_timer;
	/**
	 * The attribute initiator is used to distinguish the initiator SAI from the
	 * called SAI
	 */
	public boolean initiator;
	/**
	 * This attribute represents the maximum amount of time that CSL waits after the
	 * last connection request issued to the SAI, before issuing a new one.
	 */
	public int max_connect_timer;
	/**
	 * The maximum amount of time the CSL waits  for a message from SAI before closing
	 * the connection.
	 */
	public int max_receive_timer;
	/**
	 * The maximum amount of time in which the CSL sends a message to underlying layer
	 * SAI. When this threshold is reached, the CSL will issue a LifeSign message.
	 */
	public int max_send_timer;
	/**
	 * Accumulating time since last message received from SAI.
	 */
	private int receive_timer = 0;
	/**
	 * Accumulating time since last message.
	 */
	private int send_timer = 0;
	public SAI SAI;
	public RBC_USER RBC_USER;

	/* Begin - EA generated code for StateMachine */


	enum StateMachineEnum 
	{
		NOSTATEMACHINE,
		CSL_ENUM_CSL
	};

	enum StateEnum 
	{
		NOSTATE,
		CSL_VIRTUAL_SUBMACHINESTATE,
		CSL_ENUM_CSL_COMMS,
		CSL_ENUM_CSL_NOCOMMS
	};

	enum TransitionEnum 
	{
		NOTRANSITION,
		CSL_ENUM_NOCOMMS__TO__COMMS_237,
		CSL_ENUM_COMMS__TO__NOCOMMS_2966,
		CSL_ENUM_COMMS__TO__COMMS_244,
		CSL_ENUM_NOCOMMS__TO__NOCOMMS_236,
		CSL_ENUM_NOCOMMS__TO__NOCOMMS_346,
		CSL_ENUM_COMMS__TO__COMMS_242,
		CSL_ENUM_COMMS__TO__NOCOMMS_2965,
		CSL_ENUM_NOCOMMS__TO__COMMS_232,
		CSL_ENUM_NOCOMMS__TO__NOCOMMS_235,
		CSL_ENUM_COMMS__TO__COMMS_241,
		CSL_ENUM_COMMS__TO__COMMS_286,
		CSL_ENUM_COMMS__TO__COMMS_239,
		CSL_ENUM_COMMS__TO__COMMS_238,
		CSL_ENUM_INITIAL_168__TO__NOCOMMS_229,
		CSL_ENUM_NOCOMMS__TO__NOCOMMS_230
	};

	enum EntryEnum
	{
		NOENTRY,
		CSL_ENUM_CSL_INITIAL_168
	};

	public StateMachineImpl m_StateMachineImpl;	
	
	public CSL()
	{
		m_StateMachineImpl = null;
	}
	
	protected void finalize( ) throws Throwable
	{
	}
	
	public CSL(ContextManager pManager, String sInstanceName)
	{
		//Initialize Region Variables
		m_csl = StateEnum.NOSTATE;	
		m_sInstanceName = sInstanceName;
		m_sType = "CSL";
		m_StateMachineImpl = new StateMachineImpl(this, pManager);
	}
	
    public String m_sInstanceName;
    public String m_sType;
	
	public String GetInstanceName()
	{
		return m_sInstanceName;
	}
	
	public String GetTypeName()
	{
		return m_sType;
	}
	
	public boolean defer(Event event, StateData toState)
	{
		if (m_StateMachineImpl == null)
			return false;
		
		boolean bDefer = false;
    
		
		if(!bDefer)
		{
			if(toState.parent_state != null)
			{
				bDefer = defer(event, toState.parent_state);
			}
		}
		return bDefer;
	}

	public void TransitionProc(TransitionEnum transition, Signal signal, StateData submachineState)
	{
		if (m_StateMachineImpl == null)
			return;
		
	
		switch (transition) 
		{
			case CSL_ENUM_NOCOMMS__TO__COMMS_237:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_NOCOMMS__TO__COMMS_237", "{01C6483F-FAA3-4358-A9C4-B36F24DD726B}",m_sInstanceName);
				NOCOMMS__TO__COMMS_237(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__NOCOMMS_2966:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__NOCOMMS_2966", "{0EB15EC5-6F56-48f5-9D27-E15597FE2507}",m_sInstanceName);
				COMMS__TO__NOCOMMS_2966(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__COMMS_244:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__COMMS_244", "{131CCA21-85EE-43fd-BE52-19A8082D567A}",m_sInstanceName);
				COMMS__TO__COMMS_244(signal, submachineState); 
				break;
			case CSL_ENUM_NOCOMMS__TO__NOCOMMS_236:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_NOCOMMS__TO__NOCOMMS_236", "{2349A59E-C4F1-477f-89EF-5AFDE029C80F}",m_sInstanceName);
				NOCOMMS__TO__NOCOMMS_236(signal, submachineState); 
				break;
			case CSL_ENUM_NOCOMMS__TO__NOCOMMS_346:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_NOCOMMS__TO__NOCOMMS_346", "{2BAC5824-94F1-4c30-B773-17C8DE415364}",m_sInstanceName);
				NOCOMMS__TO__NOCOMMS_346(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__COMMS_242:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__COMMS_242", "{3B732ADA-B9AC-4be5-AEC8-D7FDF54F20E2}",m_sInstanceName);
				COMMS__TO__COMMS_242(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__NOCOMMS_2965:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__NOCOMMS_2965", "{42B92EB5-06D8-46b0-AA46-D9BE5A68DB53}",m_sInstanceName);
				COMMS__TO__NOCOMMS_2965(signal, submachineState); 
				break;
			case CSL_ENUM_NOCOMMS__TO__COMMS_232:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_NOCOMMS__TO__COMMS_232", "{52AE2EE2-D22F-4fe9-AE08-E8481F7C4C04}",m_sInstanceName);
				NOCOMMS__TO__COMMS_232(signal, submachineState); 
				break;
			case CSL_ENUM_NOCOMMS__TO__NOCOMMS_235:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_NOCOMMS__TO__NOCOMMS_235", "{56883C99-1D38-46e9-987C-0E49B3A9952D}",m_sInstanceName);
				NOCOMMS__TO__NOCOMMS_235(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__COMMS_241:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__COMMS_241", "{57DD5336-2D4B-463f-8EA7-52F14ABFF6CB}",m_sInstanceName);
				COMMS__TO__COMMS_241(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__COMMS_286:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__COMMS_286", "{694A16D2-4D12-410e-B21E-DA7F3B5F0354}",m_sInstanceName);
				COMMS__TO__COMMS_286(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__COMMS_239:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__COMMS_239", "{9D0E786B-8781-4d25-BE70-795490D36C30}",m_sInstanceName);
				COMMS__TO__COMMS_239(signal, submachineState); 
				break;
			case CSL_ENUM_COMMS__TO__COMMS_238:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_COMMS__TO__COMMS_238", "{AA930C3C-DE2D-44bb-94F8-45A059BC4E22}",m_sInstanceName);
				COMMS__TO__COMMS_238(signal, submachineState); 
				break;
			case CSL_ENUM_INITIAL_168__TO__NOCOMMS_229:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_Initial_168__TO__NOCOMMS_229", "{D04CB818-0F32-48cb-A163-568797666A6C}",m_sInstanceName);
				Initial_168__TO__NOCOMMS_229(signal, submachineState); 
				break;
			case CSL_ENUM_NOCOMMS__TO__NOCOMMS_230:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "CSL_NOCOMMS__TO__NOCOMMS_230", "{FD84300A-8068-466f-8FCA-4C1A87981A21}",m_sInstanceName);
				NOCOMMS__TO__NOCOMMS_230(signal, submachineState); 
				break;
		}
	
		m_StateMachineImpl.currentTransition.SetTransition(TransitionEnum.NOTRANSITION.ordinal(), null, "", "","");	
	}
	
	private void NOCOMMS__TO__COMMS_237_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: NOCOMMS__TO__COMMS_237");
	
		m_StateMachineImpl.m_pManager.SendEvent("RBC_USER_CONNECT_INDICATION",RBC_USER.m_StateMachineImpl);

	}

	private void NOCOMMS__TO__COMMS_237(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.EXIT, null);
		NOCOMMS__TO__COMMS_237_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__NOCOMMS_2966_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__NOCOMMS_2966");
	
		m_StateMachineImpl.m_pManager.SendEvent("RBC_USER_DISCONNECT_INDICATION",RBC_USER.m_StateMachineImpl);
		m_StateMachineImpl.m_pManager.SendEvent("SAI_DISCONNECT_REQUEST",SAI.m_StateMachineImpl);
		this.receive_timer=0;
		this.send_timer=0;

	}

	private void COMMS__TO__NOCOMMS_2966(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__NOCOMMS_2966_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__COMMS_244_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__COMMS_244");
	
		this.send_timer=0;
		m_StateMachineImpl.m_pManager.SendEvent("SAI_DATA_REQUEST.sig("+signal.parameterValues.get("arg")+")",SAI.m_StateMachineImpl);

	}

	private void COMMS__TO__COMMS_244(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__COMMS_244_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void NOCOMMS__TO__NOCOMMS_236_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: NOCOMMS__TO__NOCOMMS_236");
	
	}

	private void NOCOMMS__TO__NOCOMMS_236(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.EXIT, null);
		NOCOMMS__TO__NOCOMMS_236_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void NOCOMMS__TO__NOCOMMS_346_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: NOCOMMS__TO__NOCOMMS_346");
	
		this.connect_timer = 0;
	
		m_StateMachineImpl.m_pManager.SendEvent("SAI_CONNECT_REQUEST",SAI.m_StateMachineImpl);

	}

	private void NOCOMMS__TO__NOCOMMS_346(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.EXIT, null);
		NOCOMMS__TO__NOCOMMS_346_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__COMMS_242_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__COMMS_242");
	
		this.send_timer = this.send_timer+ 1;
		this.receive_timer = this.receive_timer+1;
	

	}

	private void COMMS__TO__COMMS_242(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__COMMS_242_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__NOCOMMS_2965_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__NOCOMMS_2965");
	
		m_StateMachineImpl.m_pManager.SendEvent("RBC_USER_DISCONNECT_INDICATION",RBC_USER.m_StateMachineImpl);
		this.receive_timer=0;
		this.send_timer=0;

	}

	private void COMMS__TO__NOCOMMS_2965(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__NOCOMMS_2965_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void NOCOMMS__TO__COMMS_232_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: NOCOMMS__TO__COMMS_232");
	
		this.connect_timer=this.max_connect_timer;
	
		m_StateMachineImpl.m_pManager.SendEvent("RBC_USER_CONNECT_INDICATION",RBC_USER.m_StateMachineImpl);

	}

	private void NOCOMMS__TO__COMMS_232(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.EXIT, null);
		NOCOMMS__TO__COMMS_232_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void NOCOMMS__TO__NOCOMMS_235_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: NOCOMMS__TO__NOCOMMS_235");
	
	}

	private void NOCOMMS__TO__NOCOMMS_235(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.EXIT, null);
		NOCOMMS__TO__NOCOMMS_235_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__COMMS_241_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__COMMS_241");
	
		this.send_timer = 0;
		this.receive_timer = this.receive_timer+1;
		
		m_StateMachineImpl.m_pManager.SendEvent("SAI_DATA_REQUEST.sig(LifeSign)",SAI.m_StateMachineImpl);

	}

	private void COMMS__TO__COMMS_241(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__COMMS_241_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__COMMS_286_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__COMMS_286");
	
		this.receive_timer=0;
		this.send_timer=0;
		m_StateMachineImpl.m_pManager.SendEvent("RBC_USER_CONNECT_INDICATION",RBC_USER.m_StateMachineImpl);

	}

	private void COMMS__TO__COMMS_286(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__COMMS_286_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__COMMS_239_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__COMMS_239");
	
		this.receive_timer = 0;
		if (signal.parameterValues.get("arg") !="LifeSign") {
		    m_StateMachineImpl.m_pManager.SendEvent("RBC_USER_DATA_INDICATION.sig("+signal.parameterValues.get("arg")+")",RBC_USER.m_StateMachineImpl);
		}

	}

	private void COMMS__TO__COMMS_239(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__COMMS_239_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void COMMS__TO__COMMS_238_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: COMMS__TO__COMMS_238");
	
	}

	private void COMMS__TO__COMMS_238(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.EXIT, null);
		COMMS__TO__COMMS_238_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_COMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void Initial_168__TO__NOCOMMS_229_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: Initial_168__TO__NOCOMMS_229");
	
		if (this.initiator) this.connect_timer=this.max_connect_timer;

	}

	private void Initial_168__TO__NOCOMMS_229(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(submachineState != null)
			submachineState.IncrementActiveCount();
		Initial_168__TO__NOCOMMS_229_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void NOCOMMS__TO__NOCOMMS_230_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: NOCOMMS__TO__NOCOMMS_230");
	
		this.connect_timer = this.connect_timer+1;

	}

	private void NOCOMMS__TO__NOCOMMS_230(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(!m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal()).IsActiveState())
		{
			return;
		}
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.EXIT, null);
		NOCOMMS__TO__NOCOMMS_230_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.CSL_ENUM_CSL_NOCOMMS, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	public boolean StateProc(StateEnum state, StateData submachineState, StateBehaviorEnum behavior, Signal signal)
	{
		return StateProc(state, submachineState, behavior, signal, EntryTypeEnum.DefaultEntry, null, 0);
	}
	
	public boolean StateProc(StateEnum state, StateData submachineState, StateBehaviorEnum behavior, Signal signal, EntryTypeEnum enumEntryType)
	{
		return StateProc(state, submachineState, behavior, signal, enumEntryType, null, 0);
	}
	
	public boolean StateProc(int state, StateData submachineState, StateBehaviorEnum behavior, Signal signal, EntryTypeEnum enumEntryType, int[] entryArray, int nArrayCount)
    {
		ArrayList<EntryEnum> entryEnumList = new ArrayList<>();
		if (entryArray != null)
		{
			for(int nEntryIndex : entryArray)
			{
				entryEnumList.add(EntryEnum.values()[nEntryIndex]);
			}
		}
        return StateProc(StateEnum.values()[state], submachineState, behavior, signal, enumEntryType, entryEnumList.toArray(new EntryEnum[entryEnumList.size()]), nArrayCount);
    }
	
	public boolean StateProc(StateEnum state, StateData submachineState, StateBehaviorEnum behavior, Signal signal, EntryTypeEnum enumEntryType, EntryEnum[] entryArray, int nArrayCount)
	{
		switch (state) 
		{
			case CSL_ENUM_CSL_COMMS:
				CSL_COMMS(behavior, submachineState, signal, enumEntryType, entryArray, nArrayCount);
				break;

			case CSL_ENUM_CSL_NOCOMMS:
				CSL_NOCOMMS(behavior, submachineState, signal, enumEntryType, entryArray, nArrayCount);
				break;
		}
		return false;
	}
	
	public boolean CSL_COMMS(StateBehaviorEnum behavior, StateData submachineState, Signal signal, EntryTypeEnum enumEntryType, EntryEnum[] entryArray, int nArrayCount) 
	{
		if (m_StateMachineImpl == null)
			return false;
	
		StateData state = m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_COMMS.ordinal());
		switch (behavior) 
		{
			case ENTRY:
				if(state.active_count > 0)
					return false;
				m_csl = StateEnum.CSL_ENUM_CSL_COMMS;
				state.IncrementActiveCount();
				CSL_COMMS_behavior(StateBehaviorEnum.ENTRY);
						
				CSL_COMMS_behavior(StateBehaviorEnum.DO);
			
				if((enumEntryType == EntryTypeEnum.EntryPointEntry || enumEntryType == EntryTypeEnum.DefaultEntry) && state.IsActiveState())
					m_StateMachineImpl.deferInternalEvent(EventProxy.EventEnum.COMPLETION, null, state);
				break;
			case EXIT:
				if(state.active_count == 0)
					return false;
				m_csl = StateEnum.NOSTATE;
				state.DecrementActiveCount();
				CSL_COMMS_behavior(StateBehaviorEnum.EXIT);
				m_StateMachineImpl.removeInternalEvent(state);
				break;
		}
	
		return true;
	}

	public boolean CSL_COMMS_behavior(StateBehaviorEnum behavior)
	{
		switch (behavior) 
		{
			case ENTRY:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Entry Behavior: CSL_COMMS");
			}
			break;
			case DO:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Do Behavior: CSL_COMMS");
			}
			break;
			case EXIT:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Exit Behavior: CSL_COMMS");
			}
			break;
		}
	
		return true;
	}
	public boolean CSL_NOCOMMS(StateBehaviorEnum behavior, StateData submachineState, Signal signal, EntryTypeEnum enumEntryType, EntryEnum[] entryArray, int nArrayCount) 
	{
		if (m_StateMachineImpl == null)
			return false;
	
		StateData state = m_StateMachineImpl.GetStateObject(submachineState, StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal());
		switch (behavior) 
		{
			case ENTRY:
				if(state.active_count > 0)
					return false;
				m_csl = StateEnum.CSL_ENUM_CSL_NOCOMMS;
				state.IncrementActiveCount();
				CSL_NOCOMMS_behavior(StateBehaviorEnum.ENTRY);
						
				CSL_NOCOMMS_behavior(StateBehaviorEnum.DO);
			
				if((enumEntryType == EntryTypeEnum.EntryPointEntry || enumEntryType == EntryTypeEnum.DefaultEntry) && state.IsActiveState())
					m_StateMachineImpl.deferInternalEvent(EventProxy.EventEnum.COMPLETION, null, state);
				break;
			case EXIT:
				if(state.active_count == 0)
					return false;
				m_csl = StateEnum.NOSTATE;
				state.DecrementActiveCount();
				CSL_NOCOMMS_behavior(StateBehaviorEnum.EXIT);
				m_StateMachineImpl.removeInternalEvent(state);
				break;
		}
	
		return true;
	}

	public boolean CSL_NOCOMMS_behavior(StateBehaviorEnum behavior)
	{
		switch (behavior) 
		{
			case ENTRY:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Entry Behavior: CSL_NOCOMMS");
			}
			break;
			case DO:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Do Behavior: CSL_NOCOMMS");
			}
			break;
			case EXIT:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Exit Behavior: CSL_NOCOMMS");
			}
			break;
		}
	
		return true;
	}
	public boolean dispatch(Event event, StateData toState, boolean bCheckOnly)
	{
		if (m_StateMachineImpl == null)
			return false;
		
		boolean bDispatched = false;
		if(toState == null || !toState.IsActiveState() && !bCheckOnly)
			return bDispatched;
		
		switch (StateEnum.values()[toState.state_enum]) 
		{
			case CSL_VIRTUAL_SUBMACHINESTATE:
				if(event.eventEnum == EventProxy.EventEnum.COMPLETION)
				{
					if(!bCheckOnly)
					{
						m_StateMachineImpl.ReleaseSubmachineState(toState);
						for (int index = m_StateMachineImpl.lstStateData.size() - 1; index >= 0; index--)
						{
							StateData state = m_StateMachineImpl.lstStateData.get(index);
							if (state == toState)
							{
								m_StateMachineImpl.lstStateData.remove(state);
								break;
							}						
						}
						//delete toState;
						toState = null;
					}				
					bDispatched = true;
				}
				break;
			case CSL_ENUM_CSL_COMMS:
				switch (event.eventEnum) 
				{
					case ENUM_TICK:
						if(this.receive_timer==this.max_receive_timer) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__NOCOMMS_2966, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						if(this.receive_timer < this.max_receive_timer && this.send_timer<this.max_send_timer) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__COMMS_242, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						if(this.receive_timer<this.max_receive_timer && this.send_timer==this.max_send_timer) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__COMMS_241, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						break;
					case ENUM_RBC_USER_DATA_REQUEST:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__COMMS_244, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_DISCONNECT_INDICATION:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__NOCOMMS_2965, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_CONNECT_INDICATION:
						if(this.initiator==false) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__COMMS_286, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						break;
					case ENUM_SAI_DATA_INDICATION:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__COMMS_239, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_ERROR_REPORT:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_COMMS__TO__COMMS_238, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
				}
				break;
			case CSL_ENUM_CSL_NOCOMMS:
				switch (event.eventEnum) 
				{
					case ENUM_SAI_CONNECT_INDICATION:
						if(this.initiator==false) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__COMMS_237, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						break;
					case ENUM_TICK:
						if(this.initiator==false) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_236, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						if(this.initiator==true && this.connect_timer==this.max_connect_timer) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_346, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						if(this.initiator==true && this.connect_timer<this.max_connect_timer) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_230, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						break;
					case ENUM_SAI_CONNECT_CONFIRM:
						if(this.initiator==true) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__COMMS_232, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						break;
					case ENUM_RBC_USER_DATA_REQUEST:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_235, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_DISCONNECT_INDICATION:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_235, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_ERROR_REPORT:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_235, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_DATA_INDICATION:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.CSL_ENUM_NOCOMMS__TO__NOCOMMS_235, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
				}
				break;
		}
		
		if (!bDispatched && toState != null && toState.parent_state != null && event.eventEnum != EventProxy.EventEnum.COMPLETION)
		{
			bDispatched = dispatch(event, toState.parent_state, true);
			if (bDispatched && !bCheckOnly)
			{
				/*1. Exit the current state; 2. Decrement the active count of the parent state; 3. dispatch the event to parent state*/
				StateProc(StateEnum.values()[toState.state_enum], toState.submachine_state, StateBehaviorEnum.EXIT, null);
				toState.parent_state.DecrementActiveCount();
				dispatch(event, toState.parent_state, false);
				event = null;
			}
		}
		
		return bDispatched;
	}
	
	StateEnum m_csl;
	
	@Override
    public void runStateMachine(String statemachine)
    {
		if(statemachine.equals("CSL_CSL"))
		{
			runStateMachine(StateMachineEnum.CSL_ENUM_CSL);
			return;
		}
    }
	
	private void runStateMachine(StateMachineEnum statemachine, StateData submachineState, Signal signal)
	{
		runStateMachine(statemachine, submachineState, signal, null, 0);
	}
	
	private void runStateMachine(StateMachineEnum statemachine, StateData submachineState, Signal signal, EntryEnum[] entryArray, int nEntryCount)
	{
		if (m_StateMachineImpl == null)
			return;
		
		if(submachineState == null)
		{
			StateInitialData initialData = new StateInitialData(StateEnum.CSL_VIRTUAL_SUBMACHINESTATE.ordinal(), StateEnum.NOSTATE.ordinal(), 1, false, "CSL_VIRTUAL_SUBMACHINESTATE", "", ""); 
			submachineState = new StateData(m_StateMachineImpl, initialData);
			submachineState.IncrementActiveCount();
			m_StateMachineImpl.lstStateData.add(submachineState);
		}
		switch (statemachine) 
		{
			case CSL_ENUM_CSL:
				{
					final int nArrayCount = 2;
					StateInitialData[] initialDataArray = 
						{
							new StateInitialData(StateEnum.CSL_ENUM_CSL_COMMS.ordinal(), StateEnum.NOSTATE.ordinal(), 0, false, "CSL_CSL_COMMS", "{C04AB919-649B-4d4d-9F53-21D883CA34D9}", m_sInstanceName),
							new StateInitialData(StateEnum.CSL_ENUM_CSL_NOCOMMS.ordinal(), StateEnum.NOSTATE.ordinal(), 0, false, "CSL_CSL_NOCOMMS", "{CE0B7B95-301D-48f7-B5CE-A00D4B5E9A61}", m_sInstanceName)
						};
		
					m_StateMachineImpl.CreateStateObjects(initialDataArray, nArrayCount, submachineState);
				}
				for(int i = 0; i < nEntryCount; i++)
				{
					switch(entryArray[i])
					{
					case CSL_ENUM_CSL_INITIAL_168:
						TransitionProc(TransitionEnum.CSL_ENUM_INITIAL_168__TO__NOCOMMS_229, signal, submachineState);
						break;
					}
				}
				if(submachineState.IsActiveState())
					m_StateMachineImpl.deferInternalEvent(EventProxy.EventEnum.COMPLETION, null, submachineState);
				break;
		}
	
	}
	
	public void runStateMachine(StateMachineEnum statemachine)
	{
		if (m_StateMachineImpl == null)
			return;
		
		List<StateData> activeStateArray = new ArrayList<StateData>();
		if(m_StateMachineImpl.GetCurrentStates(activeStateArray) > 0)
			return;
		switch (statemachine) 
		{
			case CSL_ENUM_CSL:
				{
					final int nArrayCount = 1;
					EntryEnum[] entryArray = {EntryEnum.CSL_ENUM_CSL_INITIAL_168};
					runStateMachine(statemachine, null, null, entryArray, nArrayCount);	//submachineState is NULL if run standalone statemachine 
				}
				break;
		}
	}
	
	public void runStateMachines()
	{
		runStateMachine(StateMachineEnum.CSL_ENUM_CSL);	
	}


	/* End - EA generated code for StateMachine */
}//end CSL