import java.util.*;


/**
 * The Safety Application Intermediate sub-layer class. This class is now only a
 * stub.
 * @author Davide Basile (ISTI CNR Italy)
 * @version 1.0
 * @created 23-Nov-2020 11:13:45 AM
 */
public class SAI implements StateMachineContext {

	/**
	 * The attribute initiator is used to distinguish the initiator SAI from the
	 * called SAI
	 */
	public boolean initiator;

	/* Begin - EA generated code for StateMachine */


	enum StateMachineEnum 
	{
		NOSTATEMACHINE,
		SAI_ENUM_SAI
	};

	enum StateEnum 
	{
		NOSTATE,
		SAI_VIRTUAL_SUBMACHINESTATE,
		SAI_ENUM_SAI_STUB
	};

	enum TransitionEnum 
	{
		NOTRANSITION,
		SAI_ENUM_STUB__TO__STUB_257,
		SAI_ENUM_INITIAL_157__TO__STUB_255,
		SAI_ENUM_STUB__TO__STUB_344
	};

	enum EntryEnum
	{
		NOENTRY,
		SAI_ENUM_SAI_INITIAL_157
	};

	public StateMachineImpl m_StateMachineImpl;	
	
	public SAI()
	{
		m_StateMachineImpl = null;
	}
	
	protected void finalize( ) throws Throwable
	{
	}
	
	public SAI(ContextManager pManager, String sInstanceName)
	{
		//Initialize Region Variables
		m_sai = StateEnum.NOSTATE;	
		m_sInstanceName = sInstanceName;
		m_sType = "SAI";
		m_StateMachineImpl = new StateMachineImpl(this, pManager);
	}
	
    public String m_sInstanceName;
    public String m_sType;
	
	public String GetInstanceName()
	{
		return m_sInstanceName;
	}
	
	public String GetTypeName()
	{
		return m_sType;
	}
	
	public boolean defer(Event event, StateData toState)
	{
		if (m_StateMachineImpl == null)
			return false;
		
		boolean bDefer = false;
    
		
		if(!bDefer)
		{
			if(toState.parent_state != null)
			{
				bDefer = defer(event, toState.parent_state);
			}
		}
		return bDefer;
	}

	public void TransitionProc(TransitionEnum transition, Signal signal, StateData submachineState)
	{
		if (m_StateMachineImpl == null)
			return;
		
	
		switch (transition) 
		{
			case SAI_ENUM_STUB__TO__STUB_257:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "SAI_Stub__TO__Stub_257", "{7AF0DDDF-63B3-4474-964D-99B05F3D49C8}",m_sInstanceName);
				Stub__TO__Stub_257(signal, submachineState); 
				break;
			case SAI_ENUM_INITIAL_157__TO__STUB_255:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "SAI_Initial_157__TO__Stub_255", "{B137E258-8193-4fab-8948-2C5A8517822A}",m_sInstanceName);
				Initial_157__TO__Stub_255(signal, submachineState); 
				break;
			case SAI_ENUM_STUB__TO__STUB_344:
				m_StateMachineImpl.currentTransition.SetTransition(transition.ordinal(), submachineState, "SAI_Stub__TO__Stub_344", "{D70869F9-8B5D-48e5-9F48-989C85395071}",m_sInstanceName);
				Stub__TO__Stub_344(signal, submachineState); 
				break;
		}
	
		m_StateMachineImpl.currentTransition.SetTransition(TransitionEnum.NOTRANSITION.ordinal(), null, "", "","");	
	}
	
	private void Stub__TO__Stub_257_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: Stub__TO__Stub_257");
	
	}

	private void Stub__TO__Stub_257(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		Stub__TO__Stub_257_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
	}
	private void Initial_157__TO__Stub_255_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: Initial_157__TO__Stub_255");
	
	}

	private void Initial_157__TO__Stub_255(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		if(submachineState != null)
			submachineState.IncrementActiveCount();
		Initial_157__TO__Stub_255_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
		StateProc(StateEnum.SAI_ENUM_SAI_STUB, submachineState, StateBehaviorEnum.ENTRY, signal, EntryTypeEnum.DefaultEntry);
	}
	private void Stub__TO__Stub_344_effect(Signal signal)
	{
		GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Transition Effect: Stub__TO__Stub_344");
	
	}

	private void Stub__TO__Stub_344(Signal signal, StateData submachineState) 
	{
		if (m_StateMachineImpl == null)
			return;
	
		Stub__TO__Stub_344_effect(signal);
		m_StateMachineImpl.currentTransition.SetActive(m_StateMachineImpl);
	}
	public boolean StateProc(StateEnum state, StateData submachineState, StateBehaviorEnum behavior, Signal signal)
	{
		return StateProc(state, submachineState, behavior, signal, EntryTypeEnum.DefaultEntry, null, 0);
	}
	
	public boolean StateProc(StateEnum state, StateData submachineState, StateBehaviorEnum behavior, Signal signal, EntryTypeEnum enumEntryType)
	{
		return StateProc(state, submachineState, behavior, signal, enumEntryType, null, 0);
	}
	
	public boolean StateProc(int state, StateData submachineState, StateBehaviorEnum behavior, Signal signal, EntryTypeEnum enumEntryType, int[] entryArray, int nArrayCount)
    {
		ArrayList<EntryEnum> entryEnumList = new ArrayList<>();
		if (entryArray != null)
		{
			for(int nEntryIndex : entryArray)
			{
				entryEnumList.add(EntryEnum.values()[nEntryIndex]);
			}
		}
        return StateProc(StateEnum.values()[state], submachineState, behavior, signal, enumEntryType, entryEnumList.toArray(new EntryEnum[entryEnumList.size()]), nArrayCount);
    }
	
	public boolean StateProc(StateEnum state, StateData submachineState, StateBehaviorEnum behavior, Signal signal, EntryTypeEnum enumEntryType, EntryEnum[] entryArray, int nArrayCount)
	{
		switch (state) 
		{
			case SAI_ENUM_SAI_STUB:
				SAI_Stub(behavior, submachineState, signal, enumEntryType, entryArray, nArrayCount);
				break;
		}
		return false;
	}
	
	public boolean SAI_Stub(StateBehaviorEnum behavior, StateData submachineState, Signal signal, EntryTypeEnum enumEntryType, EntryEnum[] entryArray, int nArrayCount) 
	{
		if (m_StateMachineImpl == null)
			return false;
	
		StateData state = m_StateMachineImpl.GetStateObject(submachineState, StateEnum.SAI_ENUM_SAI_STUB.ordinal());
		switch (behavior) 
		{
			case ENTRY:
				if(state.active_count > 0)
					return false;
				m_sai = StateEnum.SAI_ENUM_SAI_STUB;
				state.IncrementActiveCount();
				SAI_Stub_behavior(StateBehaviorEnum.ENTRY);
						
				SAI_Stub_behavior(StateBehaviorEnum.DO);
			
				if((enumEntryType == EntryTypeEnum.EntryPointEntry || enumEntryType == EntryTypeEnum.DefaultEntry) && state.IsActiveState())
					m_StateMachineImpl.deferInternalEvent(EventProxy.EventEnum.COMPLETION, null, state);
				break;
			case EXIT:
				if(state.active_count == 0)
					return false;
				m_sai = StateEnum.NOSTATE;
				state.DecrementActiveCount();
				SAI_Stub_behavior(StateBehaviorEnum.EXIT);
				m_StateMachineImpl.removeInternalEvent(state);
				break;
		}
	
		return true;
	}

	public boolean SAI_Stub_behavior(StateBehaviorEnum behavior)
	{
		switch (behavior) 
		{
			case ENTRY:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Entry Behavior: SAI_Stub");
			}
			break;
			case DO:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Do Behavior: SAI_Stub");
			}
			break;
			case EXIT:
			{
				GlobalFuncs.trace("[" + m_sInstanceName + ":" + m_sType + "] Exit Behavior: SAI_Stub");
			}
			break;
		}
	
		return true;
	}
	public boolean dispatch(Event event, StateData toState, boolean bCheckOnly)
	{
		if (m_StateMachineImpl == null)
			return false;
		
		boolean bDispatched = false;
		if(toState == null || !toState.IsActiveState() && !bCheckOnly)
			return bDispatched;
		
		switch (StateEnum.values()[toState.state_enum]) 
		{
			case SAI_VIRTUAL_SUBMACHINESTATE:
				if(event.eventEnum == EventProxy.EventEnum.COMPLETION)
				{
					if(!bCheckOnly)
					{
						m_StateMachineImpl.ReleaseSubmachineState(toState);
						for (int index = m_StateMachineImpl.lstStateData.size() - 1; index >= 0; index--)
						{
							StateData state = m_StateMachineImpl.lstStateData.get(index);
							if (state == toState)
							{
								m_StateMachineImpl.lstStateData.remove(state);
								break;
							}						
						}
						//delete toState;
						toState = null;
					}				
					bDispatched = true;
				}
				break;
			case SAI_ENUM_SAI_STUB:
				switch (event.eventEnum) 
				{
					case ENUM_SAI_DATA_REQUEST:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.SAI_ENUM_STUB__TO__STUB_257, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_DISCONNECT_REQUEST:
						if(!bCheckOnly)
							TransitionProc(TransitionEnum.SAI_ENUM_STUB__TO__STUB_257, event.signal, toState.submachine_state);
						bDispatched = true;
						break;
					case ENUM_SAI_CONNECT_REQUEST:
						if(this.initiator==true) 
						{
							if(!bCheckOnly)
								TransitionProc(TransitionEnum.SAI_ENUM_STUB__TO__STUB_344, event.signal, toState.submachine_state);
							bDispatched = true;
							break;
						}
						break;
				}
				break;
		}
		
		if (!bDispatched && toState != null && toState.parent_state != null && event.eventEnum != EventProxy.EventEnum.COMPLETION)
		{
			bDispatched = dispatch(event, toState.parent_state, true);
			if (bDispatched && !bCheckOnly)
			{
				/*1. Exit the current state; 2. Decrement the active count of the parent state; 3. dispatch the event to parent state*/
				StateProc(StateEnum.values()[toState.state_enum], toState.submachine_state, StateBehaviorEnum.EXIT, null);
				toState.parent_state.DecrementActiveCount();
				dispatch(event, toState.parent_state, false);
				event = null;
			}
		}
		
		return bDispatched;
	}
	
	StateEnum m_sai;
	
	@Override
    public void runStateMachine(String statemachine)
    {
		if(statemachine.equals("SAI_SAI"))
		{
			runStateMachine(StateMachineEnum.SAI_ENUM_SAI);
			return;
		}
    }
	
	private void runStateMachine(StateMachineEnum statemachine, StateData submachineState, Signal signal)
	{
		runStateMachine(statemachine, submachineState, signal, null, 0);
	}
	
	private void runStateMachine(StateMachineEnum statemachine, StateData submachineState, Signal signal, EntryEnum[] entryArray, int nEntryCount)
	{
		if (m_StateMachineImpl == null)
			return;
		
		if(submachineState == null)
		{
			StateInitialData initialData = new StateInitialData(StateEnum.SAI_VIRTUAL_SUBMACHINESTATE.ordinal(), StateEnum.NOSTATE.ordinal(), 1, false, "SAI_VIRTUAL_SUBMACHINESTATE", "", ""); 
			submachineState = new StateData(m_StateMachineImpl, initialData);
			submachineState.IncrementActiveCount();
			m_StateMachineImpl.lstStateData.add(submachineState);
		}
		switch (statemachine) 
		{
			case SAI_ENUM_SAI:
				{
					final int nArrayCount = 1;
					StateInitialData[] initialDataArray = 
						{
							new StateInitialData(StateEnum.SAI_ENUM_SAI_STUB.ordinal(), StateEnum.NOSTATE.ordinal(), 0, false, "SAI_SAI_Stub", "{DA84CC61-AFEA-4c01-BE9C-3B8A9341A64E}", m_sInstanceName)
						};
		
					m_StateMachineImpl.CreateStateObjects(initialDataArray, nArrayCount, submachineState);
				}
				for(int i = 0; i < nEntryCount; i++)
				{
					switch(entryArray[i])
					{
					case SAI_ENUM_SAI_INITIAL_157:
						TransitionProc(TransitionEnum.SAI_ENUM_INITIAL_157__TO__STUB_255, signal, submachineState);
						break;
					}
				}
				if(submachineState.IsActiveState())
					m_StateMachineImpl.deferInternalEvent(EventProxy.EventEnum.COMPLETION, null, submachineState);
				break;
		}
	
	}
	
	public void runStateMachine(StateMachineEnum statemachine)
	{
		if (m_StateMachineImpl == null)
			return;
		
		List<StateData> activeStateArray = new ArrayList<StateData>();
		if(m_StateMachineImpl.GetCurrentStates(activeStateArray) > 0)
			return;
		switch (statemachine) 
		{
			case SAI_ENUM_SAI:
				{
					final int nArrayCount = 1;
					EntryEnum[] entryArray = {EntryEnum.SAI_ENUM_SAI_INITIAL_157};
					runStateMachine(statemachine, null, null, entryArray, nArrayCount);	//submachineState is NULL if run standalone statemachine 
				}
				break;
		}
	}
	
	public void runStateMachines()
	{
		runStateMachine(StateMachineEnum.SAI_ENUM_SAI);	
	}


	/* End - EA generated code for StateMachine */
}//end SAI